@@ -7,8 +7,13 @@
 )]
 #![doc = include_str!("../README.md")]
 #![warn(
+    clippy::integer_arithmetic,
+    clippy::panic,
+    clippy::panic_in_result_fn,
+    clippy::unwrap_used,
     missing_docs,
     rust_2018_idioms,
+    unsafe_code,
     unused_lifetimes,
     unused_qualifications
 )]
